Stan Math Library: stan/math/prim/scal/prob/uniform_cdf.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

uniform_cdf.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_UNIFORM_CDF_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_UNIFORM_CDF_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_greater.hpp>
10 #include <stan/math/prim/scal/err/check_not_nan.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/value_of.hpp>
13 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
14 #include <stan/math/prim/scal/meta/include_summand.hpp>
15 #include <boost/random/uniform_real_distribution.hpp>
16 #include <boost/random/variate_generator.hpp>
17
18 namespace stan {
19
20 namespace math {
21
22 template <typename T_y, typename T_low, typename T_high>
23 typename return_type<T_y, T_low, T_high>::type
24 uniform_cdf(const T_y& y, const T_low& alpha, const T_high& beta) {
25 static const char* function("stan::math::uniform_cdf");
26 typedef typename stan::partials_return_type<T_y, T_low, T_high>::type
27 T_partials_return;
28
29 using stan::math::check_not_nan;
30 using stan::math::check_finite;
31 using stan::math::check_greater;
32 using stan::math::value_of;
33 using stan::math::check_consistent_sizes;
34
35 // check if any vectors are zero length
36 if (!(stan::length(y)
37 && stan::length(alpha)
38 && stan::length(beta)))
39 return 1.0;
40
41 // set up return value accumulator
42 T_partials_return cdf(1.0);
43 check_not_nan(function, "Random variable", y);
44 check_finite(function, "Lower bound parameter", alpha);
45 check_finite(function, "Upper bound parameter", beta);
46 check_greater(function, "Upper bound parameter", beta, alpha);
47 check_consistent_sizes(function,
48 "Random variable", y,
49 "Lower bound parameter", alpha,
50 "Upper bound parameter", beta);
51
52 VectorView<const T_y> y_vec(y);
53 VectorView<const T_low> alpha_vec(alpha);
54 VectorView<const T_high> beta_vec(beta);
55 size_t N = max_size(y, alpha, beta);
56
57 for (size_t n = 0; n < N; n++) {
58 const T_partials_return y_dbl = value_of(y_vec[n]);
59 if (y_dbl < value_of(alpha_vec[n])
60 || y_dbl > value_of(beta_vec[n]))
61 return 0.0;
62 }
63
64 OperandsAndPartials<T_y, T_low, T_high>
65 operands_and_partials(y, alpha, beta);
66 for (size_t n = 0; n < N; n++) {
67 const T_partials_return y_dbl = value_of(y_vec[n]);
68 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
69 const T_partials_return beta_dbl = value_of(beta_vec[n]);
70 const T_partials_return b_min_a = beta_dbl - alpha_dbl;
71 const T_partials_return cdf_ = (y_dbl - alpha_dbl) / b_min_a;
72
73 // cdf
74 cdf *= cdf_;
75
76 // gradients
77 if (!is_constant_struct<T_y>::value)
78 operands_and_partials.d_x1[n] += 1.0 / b_min_a / cdf_;
79 if (!is_constant_struct<T_low>::value)
80 operands_and_partials.d_x2[n] += (y_dbl - beta_dbl) / b_min_a
81 / b_min_a / cdf_;
82 if (!is_constant_struct<T_high>::value)
83 operands_and_partials.d_x3[n] -= 1.0 / b_min_a;
84 }
85
86 if (!is_constant_struct<T_y>::value) {
87 for (size_t n = 0; n < stan::length(y); ++n)
88 operands_and_partials.d_x1[n] *= cdf;
89 }
90 if (!is_constant_struct<T_low>::value) {
91 for (size_t n = 0; n < stan::length(alpha); ++n)
92 operands_and_partials.d_x2[n] *= cdf;
93 }
94 if (!is_constant_struct<T_high>::value) {
95 for (size_t n = 0; n < stan::length(beta); ++n)
96 operands_and_partials.d_x3[n] *= cdf;
97 }
98
99 return operands_and_partials.value(cdf);
100 }
101 }
102 }
103 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
include_summand.hpp
check_greater.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
constants.hpp
stan::math::uniform_cdf
return_type< T_y, T_low, T_high >::type uniform_cdf(const T_y &y, const T_low &alpha, const T_high &beta)
Definition: uniform_cdf.hpp:24
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_greater
bool check_greater(const char *function, const char *name, const T_y &y, const T_low &low)
Return true if y is strictly greater than low.
Definition: check_greater.hpp:84
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.